@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 from arkindex_cli.commands.elements.link import add_linking_parser
 from arkindex_cli.commands.elements.list import add_list_parser
+from arkindex_cli.commands.elements.ml_splits import add_splits_parser
 from arkindex_cli.commands.elements.page_copy import add_page_copy_parser
 from arkindex_cli.commands.elements.reject_classifications import (
     add_classification_rejection_parser,
@@ -20,6 +21,7 @@ def add_elements_parser(subcommands) -> None:
     add_page_copy_parser(subparsers)
     add_classification_rejection_parser(subparsers)
     add_list_parser(subparsers)
+    add_splits_parser(subparsers)
 
     def subcommand_required(*args, **kwargs):
         elements_parser.error("A subcommand is required.")
