@@ -2,6 +2,8 @@
 from pathlib import Path
 
 from arkindex_cli.commands.export.alto import add_alto_parser
+from arkindex_cli.commands.export.csv import add_csv_parser
+from arkindex_cli.commands.export.entities import add_entities_parser
 from arkindex_cli.commands.export.pdf import add_pdf_parser
 
 
@@ -20,6 +22,8 @@ def add_export_parser(subcommands) -> None:
     subparsers = export_parser.add_subparsers()
     add_pdf_parser(subparsers)
     add_alto_parser(subparsers)
+    add_csv_parser(subparsers)
+    add_entities_parser(subparsers)
 
     def subcommand_required(*args, **kwargs):
         export_parser.error("A subcommand is required.")
