@@ -11,12 +11,12 @@ from xml.sax.saxutils import escape
 from arkindex_cli.commands.export.db import (
     Element,
     element_image,
-    element_transcriptions,
     filter_folder_id,
     get_elements,
     get_worker_version,
     list_children,
     list_folders,
+    recursive_element_transcriptions,
 )
 from arkindex_cli.commands.export.utils import BoundingBox
 
@@ -184,7 +184,9 @@ def alto_xml_gen(
                 alto_page.set("PROCESSINGREFS", f"worker_{page.worker_version_id}")
 
             # getting the list of Transcriptions namedtuples for each page
-            page_transcriptions = element_transcriptions(database_path, page.id)
+            page_transcriptions = recursive_element_transcriptions(
+                database_path, page.id
+            )
 
             # creating GraphicalElement tags for layout_tag elements if specified
             if layout_tag:
