@@ -13,6 +13,8 @@ Element = namedtuple("Element", ["id", "name", "polygon", "worker_version_id"])
 TypedElement = namedtuple(
     "Element", ["id", "name", "type", "polygon", "worker_version_id", "created"]
 )
+ElementEntity = namedtuple("ElementEntity", ["entity_type", "name", "number"])
+ElementEntity.__doc__ += "Only the type and name of an entity attached to an element through a transcription."
 Transcription = namedtuple(
     "Transcription", ["id", "element_id", "text", "confidence", "worker_version_id"]
 )
@@ -24,7 +26,7 @@ TranscriptionEntity = namedtuple(
         "entity_id",
         "entity_type",
         "entity_value",
-        "entity_subtype",
+        "entity_metas",
         "offset",
         "length",
     ],
@@ -33,7 +35,13 @@ TranscriptionEntity.__doc__ += "This does not only contain data from the Transcr
 Worker = namedtuple("Worker", ["id", "name", "slug", "type", "revision"])
 Metadata = namedtuple("Metadata", ["name", "value", "number"])
 Classification = namedtuple(
-    "Classification", ["class_name", "class_id", "classification_confidence"]
+    "Classification",
+    [
+        "class_name",
+        "class_id",
+        "classification_confidence",
+        "classification_worker_version",
+    ],
 )
 
 
@@ -314,10 +322,7 @@ def classes_columns(
                 SELECT DISTINCT class_name FROM classification
                 WHERE element_id IN element_ids
                 """,
-                (
-                    str(parent_id),
-                    element_type,
-                ),
+                (str(parent_id), element_type),
             )
         else:
             cursor.execute(
@@ -330,10 +335,7 @@ def classes_columns(
                 SELECT DISTINCT class_name FROM classification
                 WHERE element_id IN element_ids
                 """,
-                (
-                    str(parent_id),
-                    element_type,
-                ),
+                (str(parent_id), element_type),
             )
     elif element_type:
         cursor.execute(
@@ -376,7 +378,7 @@ def classes_columns(
                 SELECT DISTINCT class_name FROM classification
                 WHERE element_id IN element_ids
                 """,
-                (str(parent_id),),
+                (str(parent_id)),
             )
     else:
         cursor.execute(
@@ -556,7 +558,9 @@ def metadata_columns(
 
 
 def element_classes(
-    database_path: str, element_id: str
+    database_path: str,
+    element_id: str,
+    classification_worker_version: Optional[uuid.UUID],
 ) -> Optional[List[Classification]]:
     """
     Retrieve an element's classes.
@@ -565,14 +569,31 @@ def element_classes(
     connection = sqlite3.Connection(database_path)
     cursor = connection.cursor()
 
-    cursor.execute(
-        """
-        SELECT class_name, id, confidence FROM classification
-        WHERE element_id = ?;
-        """,
-        (element_id,),
-    )
+    if classification_worker_version == "manual":
+        cursor.execute(
+            """
+            SELECT class_name, id, confidence, worker_version_id FROM classification
+            WHERE element_id = ? AND classification.worker_version_id IS NULL;
+            """,
+            (element_id,),
+        )
 
+    elif classification_worker_version:
+        cursor.execute(
+            """
+            SELECT class_name, id, confidence, worker_version_id FROM classification
+            WHERE element_id = ? AND classification.worker_version_id = ?;
+            """,
+            (element_id, classification_worker_version),
+        )
+    else:
+        cursor.execute(
+            """
+            SELECT class_name, id, confidence, worker_version_id FROM classification
+            WHERE element_id = ?;
+            """,
+            (element_id,),
+        )
     result = cursor.fetchall()
     cursor.close()
     connection.close()
@@ -583,7 +604,7 @@ def element_classes(
 
 def transcription_entities(database_path: str) -> Optional[List[TranscriptionEntity]]:
     """
-    Retrieve a transcription's entities
+    Retrieve all transcription entities in the corpus
     """
 
     connection = sqlite3.Connection(database_path)
@@ -591,9 +612,12 @@ def transcription_entities(database_path: str) -> Optional[List[TranscriptionEnt
 
     cursor.execute(
         """
-        SELECT transcription_id, transcription.element_id, entity_id, entity.type, entity.name, entity.metas, offset, length FROM transcription_entity
+        SELECT transcription_id, transcription.element_id, entity_id, entity_type.name, entity.name, entity.metas, offset, length
+        FROM transcription_entity
+        INNER JOIN entity_type ON entity.type_id = entity_type.id
         INNER JOIN entity ON entity_id = entity.id
         INNER JOIN transcription ON transcription_id = transcription.id
+        ORDER BY transcription_id, entity_id
         """,
     )
 
@@ -634,7 +658,7 @@ def element_image(database_path: str, element_id: str) -> Optional[Image]:
     return Image(*result)
 
 
-def element_transcriptions(
+def recursive_element_transcriptions(
     database_path: str, element_id: str
 ) -> Optional[Transcription]:
 
@@ -656,7 +680,7 @@ def element_transcriptions(
         )
         SELECT
         transcription.id, transcription.element_id, text, confidence, worker_version_id
-        FROM child_ids LEFT JOIN transcription
+        FROM child_ids INNER JOIN transcription
         ON child_ids.element_id = transcription.element_id;
         """,
         (element_id,),
@@ -670,6 +694,238 @@ def element_transcriptions(
     return list(starmap(Transcription, result))
 
 
+def element_transcription(
+    database_path: str, element_id: str
+) -> Optional[Transcription]:
+
+    # connection to the database
+    connection = sqlite3.Connection(database_path)
+    cursor = connection.cursor()
+
+    # query execution
+    cursor.execute(
+        """
+        SELECT id, element_id, text, confidence, worker_version_id FROM transcription
+        WHERE element_id = ?
+        """,
+        (element_id,),
+    )
+
+    result = cursor.fetchall()
+    cursor.close()
+    connection.close()
+    if result is None:
+        return
+    return list(starmap(Transcription, result))
+
+
+def element_entities(
+    database_path: str,
+    element_id: str,
+    worker_version_id: str = None,
+    entities_recursive: bool = False,
+) -> Optional[List[ElementEntity]]:
+
+    connection = sqlite3.Connection(database_path)
+    cursor = connection.cursor()
+
+    params = [
+        element_id,
+    ]
+    where = ""
+
+    if entities_recursive:
+        base_query = """
+        WITH RECURSIVE child_ids (element_id) AS (
+            SELECT ?
+                UNION
+                    SELECT child_id
+                    FROM element_path
+                    WHERE parent_id = ?
+                UNION
+                    SELECT child_id
+                    FROM element_path
+                    JOIN child_ids ON (element_path.parent_id = child_ids.element_id)
+        )
+        SELECT entity_type.name, entity.name, ROW_NUMBER() OVER (PARTITION BY entity_type.name) AS number
+        FROM child_ids INNER JOIN transcription ON child_ids.element_id = transcription.element_id
+        """
+        params.append(element_id)
+    else:
+        base_query = """
+        SELECT entity_type.name, entity.name, ROW_NUMBER() OVER (PARTITION BY entity_type.name) AS number
+        FROM transcription
+        """
+        where = "WHERE transcription.element_id = ?"
+
+    kw = "WHERE"
+    if where:
+        kw = "AND"
+    if worker_version_id == "manual":
+        where = " ".join(
+            [where, f"{kw} transcription_entity.worker_version_id IS NULL"]
+        ).strip()
+    elif worker_version_id is not None:
+        where = " ".join(
+            [where, f"{kw} transcription_entity.worker_version_id = ?"]
+        ).strip()
+        params.append(worker_version_id)
+
+    query = f"""
+        {base_query}
+        INNER JOIN transcription_entity ON transcription_entity.transcription_id = transcription.id
+        INNER JOIN entity ON entity.id = transcription_entity.entity_id
+        INNER JOIN entity_type ON entity_type.id = entity.type_id
+        {where}
+        GROUP BY entity.name
+        ORDER BY entity_type.name, entity.name;
+        """
+
+    params = list(map(str, params))
+
+    cursor.execute(query, params)
+
+    result = cursor.fetchall()
+    cursor.close()
+    connection.close()
+    if result is None:
+        return []
+    return list(starmap(ElementEntity, result))
+
+
+def entity_type_columns(
+    database_path: str,
+    parent_id: Optional[uuid.UUID],
+    element_type: Optional[str],
+    entities_worker_version: Optional[str],
+    recursive: False,
+) -> List[str]:
+    connection = sqlite3.Connection(database_path)
+    cursor = connection.cursor()
+
+    params = []
+    join = """
+            FROM (
+                SELECT transcription.element_id, entity_type.name, COUNT(*) AS count
+                FROM element_ids INNER JOIN transcription ON element_ids.id = transcription.element_id
+        """
+
+    if parent_id:
+        params = [parent_id, parent_id]
+        if recursive:
+            base_query = """
+                WITH RECURSIVE child_ids (element_id) AS (
+                    SELECT ?
+                UNION
+                    SELECT child_id
+                    FROM element_path
+                    WHERE parent_id = ?
+                UNION
+                    SELECT child_id
+                    FROM element_path
+                    JOIN child_ids ON (element_path.parent_id = child_ids.element_id)
+                )
+            """
+        else:
+            base_query = """
+                WITH element_ids (id) AS (
+                    SELECT ?
+                UNION
+                    SELECT element.id
+                    FROM element
+                    LEFT JOIN element_path ON (element.id = element_path.child_id)
+                    WHERE element.id IN (SELECT child_id FROM element_path WHERE parent_id = ?)
+                )
+            """
+        if element_type:
+            if recursive:
+                base_query = f"""
+                    {base_query}
+                    , element_ids (id) AS (
+                        SELECT element.id FROM element JOIN child_ids ON element.id = element_id WHERE type = ?
+                    )
+                    """
+            else:
+                base_query = f"""
+                    {base_query}
+                    , final_ids (id) AS (
+                        SELECT element.id FROM element JOIN element_ids ON element.id = element_ids.id WHERE type = ?
+                    )
+                    """
+                join = """
+                    FROM (
+                        SELECT transcription.element_id, entity_type.name, COUNT(*) AS count
+                        FROM final_ids INNER JOIN transcription ON final_ids.id = transcription.element_id
+                    """
+            params.append(element_type)
+    else:
+        if element_type:
+            base_query = """
+                WITH element_ids as (SELECT element.id
+                    FROM element
+                    WHERE type = ?)
+            """
+            params.append(element_type)
+        else:
+            base_query = ""
+            join = """
+                FROM (
+                    SELECT transcription.element_id, entity_type.name, COUNT(*) AS count
+                    FROM element INNER JOIN transcription ON element.id = transcription.element_id
+            """
+
+    if parent_id and recursive and not element_type:
+        join = """
+                FROM (
+                    SELECT transcription.element_id, entity_type.name, COUNT(*) AS count
+                    FROM child_ids INNER JOIN transcription ON child_ids.element_id = transcription.element_id
+            """
+
+    if entities_worker_version == "manual":
+        where = "WHERE transcription_entity.worker_version_id IS NULL"
+    elif entities_worker_version is not None:
+        where = "WHERE transcription_entity.worker_version_id = ?"
+        params.append(entities_worker_version)
+    else:
+        where = ""
+
+    query = f"""
+        {base_query}
+        SELECT subquery.name, MAX(count)
+        {join}
+        INNER JOIN transcription_entity ON transcription_entity.transcription_id = transcription.id
+        INNER JOIN entity ON entity.id = transcription_entity.entity_id
+        INNER JOIN entity_type ON entity_type.id = entity.type_id
+        {where}
+        GROUP BY entity_type.name
+        ) subquery GROUP BY name;
+    """
+
+    params = list(map(str, params))
+
+    cursor.execute(query, params)
+    result = cursor.fetchall()
+
+    if result is None:
+        return []
+    columns = []
+    for name, count in result:
+        if count > 1:
+            for i in range(1, count + 1):
+                column_name = f"entity_{name}_{i}"
+                assert (
+                    column_name not in columns
+                ), f"Duplicate entity type column: {column_name}."
+                columns.append(column_name)
+                i += 1
+        else:
+            if name not in columns:
+                columns.append(f"entity_{name}")
+    cursor.close()
+    connection.close()
+    return columns
+
+
 def get_worker_version(database_path: str, worker_version_id: str) -> Optional[Worker]:
     """
     Returns the worker namedtuple from the given element id
