@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 import logging
+import os
 from typing import Optional
 
 from arkindex import ArkindexClient
@@ -12,8 +13,8 @@ from arkindex_cli.commands.models.utils import (
     create_or_retrieve_model,
     find_model_path,
     parse_yml_config,
-    update_model_version,
     upload_to_s3,
+    validate_model_version,
 )
 
 logging.basicConfig(level=logging.INFO, format="[%(levelname)s] %(message)s")
@@ -63,6 +64,15 @@ def publish_model(
     # On 403 abort and log error
     model_id = create_or_retrieve_model(client=client, name=name)
 
+    # Create a version for this model
+    model_version = create_model_version(
+        client=client,
+        model_id=model_id,
+        tag=os.environ.get("CI_COMMIT_TAG"),
+        description=name,
+        configuration=configuration,
+    )
+
     # Create the zst archive, get its hash and size
     with create_archive(
         path=path_to_model, rename=rename, use_parent_folder=use_parent_folder
@@ -72,27 +82,22 @@ def publish_model(
         size,
         archive_hash,
     ):
-        # Create a new model version with hash and size
-        model_version_details = create_model_version(
+        upload_to_s3(archive_path=path_to_archive, model_version_details=model_version)
+
+        # Validate the model version with hash, archive_hash and size
+        valid_version = validate_model_version(
             client=client,
-            model_id=model_id,
-            hash=hash,
+            model_version_id=model_version["id"],
             size=size,
+            hash=hash,
             archive_hash=archive_hash,
         )
-        if model_version_details is None:
-            return
-        upload_to_s3(
-            archive_path=path_to_archive, model_version_details=model_version_details
-        )
 
-    # Update the model version with state, configuration parsed, tag, description (defaults to name of the worker)
-    update_model_version(
-        client=client,
-        model_version_details=model_version_details,
-        name=name,
-        configuration=configuration,
-    )
+    if valid_version["id"] != model_version["id"]:
+        logger.warning(
+            f"Model version {model_version['id']} has been marked as erroneous and left aside."
+            f"Using existing model version {valid_version['id']}"
+        )
 
 
 def run(
