@@ -6,12 +6,14 @@ import tarfile
 import tempfile
 from contextlib import contextmanager
 from pathlib import Path
-from typing import NewType, Optional, Tuple
+from typing import NewType, Optional, Tuple, Union
+from uuid import UUID
 
 import apistar
 import requests
 import yaml
 import zstandard as zstd
+from apistar.exceptions import ErrorResponse
 from arkindex import ArkindexClient
 
 logger = logging.getLogger(__name__)
@@ -27,6 +29,13 @@ Archive = Tuple[FilePath, Hash, FileSize]
 DEFAULT_MODEL_DIR = "/usr/share/teklia/"
 
 
+def build_clean_payload(**kwargs):
+    """
+    Remove null attributes from an API body payload
+    """
+    return {key: value for key, value in kwargs.items() if value is not None}
+
+
 def find_model_path(config_model_path: str, use_parent_folder: bool = False) -> str:
     """We try the following paths in that order and return the first that exists
     - /usr/share/teklia/${config_model_path}
@@ -71,22 +80,23 @@ def create_archive(
     # Create an uncompressed tar archive with all the needed files
     # Files hierarchy is kept in the archive.
     with tarfile.open(path_to_tar_archive, "w") as tar:
-        if rename:
-            assert os.path.isfile(
-                path
-            ), "This option cannot be used when processing multiple files"
+        if use_parent_folder:
+            if rename is not None:
+                # Publish parent folder under new name
+                tar.add(path, arcname=f"./{rename}")
+            else:
+                # Only publish contents of the given folder without parent folder
+                tar.add(path, arcname=".")
+        elif rename is not None:
             # Rename the single file to the given name
             tar.add(path, arcname=rename)
         else:
-            if use_parent_folder:
-                # Only publish contents of the given folder without parent folder
-                tar.add(path, arcname=".")
-            else:
-                tar.add(path)
+            tar.add(path)
         file_list = [member for member in tar.getnames() if os.path.isfile(member)]
 
     # Sort by path
     file_list.sort()
+
     # Compute hash of the files
     for file_path in file_list:
         with open(file_path, "rb") as file_data:
@@ -186,25 +196,24 @@ def parse_yml_config(worker_config_path: str, single_worker: bool = False) -> di
 
 
 def create_model_version(
-    client: ArkindexClient, model_id: str, hash: str, size: int, archive_hash: str
+    client: ArkindexClient,
+    model_id: str,
+    tag: Optional[str] = None,
+    description: Optional[str] = None,
+    configuration: Optional[dict] = {},
+    parent: Optional[Union[str, UUID]] = None,
 ) -> dict:
-    # Create a new model version with hash and size
-    try:
-        model_version_details = client.request(
-            "CreateModelVersion",
-            id=model_id,
-            body={"hash": hash, "size": size, "archive_hash": archive_hash},
-        )
-    except apistar.exceptions.ErrorResponse as e:
-        if e.status_code >= 500:
-            logger.error(f"Failed to create model version: {e.content}")
-        model_version_details = e.content.get("hash")
-        # If the existing model is in Created state, this model is returned as a dict.
-        # Else an error in a list is returned.
-        if model_version_details and isinstance(model_version_details, (list, tuple)):
-            logger.error(model_version_details[0])
-            return
-    return model_version_details
+    # Create a new model version with optional base attributes
+    return client.request(
+        "CreateModelVersion",
+        id=model_id,
+        body=build_clean_payload(
+            tag=tag,
+            description=description,
+            configuration=configuration,
+            parent=parent,
+        ),
+    )
 
 
 def upload_to_s3(archive_path: str, model_version_details: dict) -> None:
@@ -220,20 +229,25 @@ def upload_to_s3(archive_path: str, model_version_details: dict) -> None:
     r.raise_for_status()
 
 
-def update_model_version(
-    client: ArkindexClient, model_version_details: dict, name: str, configuration: dict
-) -> None:
-    logger.info("Updating the model version...")
+def validate_model_version(
+    client: ArkindexClient,
+    model_version_id: Union[str, UUID],
+    hash: str,
+    size: int,
+    archive_hash: str,
+) -> dict:
+    # Sets the model version as `Available`, once its archive has been uploaded to S3.
+    logger.info("Validating the model version...")
     try:
-        client.request(
-            "UpdateModelVersion",
-            id=model_version_details.get("id"),
-            body={
-                "state": "available",
-                "description": name,
-                "tag": os.environ.get("CI_COMMIT_TAG"),
-                "configuration": configuration,
-            },
+        return client.request(
+            "ValidateModelVersion",
+            id=model_version_id,
+            body={"size": size, "hash": hash, "archive_hash": archive_hash},
         )
-    except apistar.exceptions.ErrorResponse as e:
-        logger.error(f"Failed to update model version: {e.content}")
+    except ErrorResponse as e:
+        msg = getattr(e, "content", repr(e))
+        if e.status_code == 409:
+            logger.warning(f"An available model version exists with hash {hash}")
+            return e.content
+        logger.error(f"Failed to update model version: {msg}")
+        raise e
